#!/usr/bin/perl -w

my $prog_name = "30_DiagHunter.pl";
my $VERSION = 1.17;

use strict;
use GD; # make PNG images 
# GD is available from CPAN, or on Randy Kobes site for ActiveState (PC)
use File::Spec::Functions; # handle path separators on different platforms

# Copyright Steven Cannon, 2003 (see specific notice in POD)
# cann0010@umn.edu  

# POD documentation
#------------------------------------------------------------------------------#
=pod BEGIN

=head1 NAME

 30_DiagHunter.pl

=head1 SYNOPSIS

 30_DiagHunter.pl ../cfg/30_DH.cfg

   See the sample config_file for parameters, parameter descriptions,
   and defaults, and 'pod2man 30_DiagHunter.pl | nroff -man | more'
   for more documentation.
   (c) Steven Cannon, 2003   cann0010@umn.edu

   Options are set in a config file:
     near_main_diag  exclude hits close to main diag in selfxself. Use ~ 1 - 25
     file_list       input file(s), separated by space(s) or tab(s)
     config_file     config filename
     base_dir        directory where all work will be done
     in_subdir       subdirectory path in base_dir
     out_subdir      subdirectory path in base_dir (will be created)
     compress_factor how much to compress orig. matrix. Increases sensitivity.
     use_orientation use gene orientation? Requires input with strand (w/c) info
     score_cutoff    bit score (or other score) to accept in diagonals
     sensitivity     how far to look in extending diags. 0 to 75, low to high.
     min_diag_len    length of a diagonal, before it is reported 
     min_diag_qual   average density required to extend a diagonal. 1 to ~10.
     plot_scale_factor factor to change the size of the dot plot; e.g. 0.5
     ids_on_png      print diagonal names on big dot plot?
     debug           amount of info to print

=head1 DESCRIPTION

 This program takes parsed Athal blast hits, in format X Y bitscore
 and spits out two dot plot png files, and a text file with coordinates
 of diagonals in the dot plots

 NOTE: The DiagHunter distribution at http://www.tc.umn.edu/~cann0010 includes 
 a BioPerl-based Blast controller and parser, additional pre-processing scripts 
 to handle other data sources, and additional sample data. Predictions of 
 diagonal (duplicated or colinear) regions in sample data are significantly 
 more sensitive in the http://www.tc.umn.edu/~cann0010 distribution than at 
 http://www.atgc.org/GenoPix_2D_Plotter/ , due to some differences in 
 source data and pre-processing steps. Otherwise, the DiagHunter 
 programs are the same in both distributions.

  Note about images:

  For graphics output (.png files), GD.pm is required. The .png plots use 
  this color scheme:
  
       red     both genes are w/w or c/c orientation (or if no w/c info)
       violet  both genes are w/c or c/w orientation
       black   (small plot): diagonal
       green   (large plot): diagonal (numbered to match text output
               in .coords and diagonal_group_*.txt files)

=head1 OPTIONS

All options are set in the configuration file:

near_main_diag: 20
  
  Exclude hits near self, in comparison of a chromosome vs. itself. Values
  of 1 to ~10 are reasonable, if working with 1k chunks

base_dir: ../Athal

  Directory where all work will be done (and where other subdirectories 
  will be created). Can be absolute or relative.

in_subdir: directory_containing_input_files
  
  Subdirectory in base_dir that contains the input files

out_subdir: directory_to_contain_output_files
  
  Subdirectory in base_dir to contain the output files
  (actually, specify the base name for the directory; a new directory
  containing some parameter information will be created).

file_list: chr1x1.bls chr2x2.bls...

  Input filenames, separated by ' ' if more than one. File name(s) should  
  have the format /[a-z]+\d+x\d+\..+/, for example 'chr2x4.aaBlsParsd' or 
  'chr1x1.aaTest'. Each file should contain the similarity results for 
  one chromosome pair. These names will be used to generate output 
  filenames, to which are added some parameter information (the bit score 
  threshold, compress_factor, and min_diag_len.
  Data in this file should be in the form
  ID1 psn1w ID2 psn2c ## ## len
  where 'w' and 'c' in the position fields refer to "Watson" (5'->3') 
  or "Crick" (3'->5') orientation of the gene.
  e.g.
  NP_171123.1	5630w	NP_564456.1	11913672w	0.42	21	131
  NP_171234.1	12940c	NP_564567.1	25481291w	0.53	40	154
  NP_171345.1	12940w	NP_172678.1	4543417c	0.62	40	129
  or 
  At1g12340	2484w	At1g12350	2484c	1	200	1626
  At1g12360	2484c	At1g12370	2486c	1	136	1222
  At1g12380	2486w	At1g12390	2486w	1	179	1598
  
  In the first example, the second and fourth fields contain gene positions
  in nucleotide coordinates, so compress_factor should be ~10000 to 30000,
  and the last field contains hit length. The
  score_cutoff is based on the last field (length).
  
  In the second example, the second and fourth fields contain gene positions
  in kb, so compress_factor should be ~10 to 30, and the last field 
  contains bit score. The score_cutoff is based on the last field (length).

use_orientation: 1

  This flag indicates whether to use the Watson (5' -> 3') and Crick (3' -> 5') 
  transcription orientation from the gene predictions. This is stored, e.g.
  At1g01010	4w	At1g01010	4w	1	200	2233
  At1g01010	4w	At1g02250	438c	1	30	323
  The strand information is in the 'w' or 'c' appended to the gene positions. 
  This format is produced by my BlastAndParse script

score_column: 5

  Specifies which column holds the score to use in determining whether to 
  accept blast hits. The score can be the bit score, a normalized expectation
  value, or other computed score.

score_cutoff: 100

  A score threshold for accepting a "hit" as a member of a diagonal. Might
  be bit score (e.g. 300 for BlastX), or .4 for a normalized [0,1] E-value, etc.

sensitivity: 75

  How far to stretch in looking for new elements in the diagonal. 0 means
  look just in the immediate diagonal; 2 means look up to two positions
  away; at 5 the search extends up to 4 positions away, as well as one
  position off-diagonal; and by 75, the search extends up to 9 positions
  away along the diagonal, and up to 5 positions above or below the diagonal.
  In effect, this value sets the size of a "cloud" of potential positions
  that might be searched, in order. See internal documentation or 
  pod2man 30_DiagHunter.pl | nroff -man | more 
  for more details.
  
min_diag_len: 5

  Length of a diagonal (numbers of hits) before it is reported. 4 is probably 
  minimum.

min_diag_qual: 10

  The min_diag_qual is the threshold for accepting the average "quality" of 
  a diagonal. Lower is more stringent.  Reasonable ranges are about 8 to 20. 
  Quality is measured as (average sensitivity)^.8, where sensitivity values 
  range from 1 to 75. So the most restrictive parameter would be 1, and
  the least restrictive would be about 32. 

compress_factor: 20000

  The compress_factor is how much the original matrix should be compressed. 
  Total compression should be at least 1000, because it would be very
  unusual for the midpoints of two genes to fall within the same 1kb. 
  Compression beyond this increases the sensitivity (at the cost of 
  selectivity), because it allows the algorithm to better span small gaps 
  in a diagonal. For Arabidopsis chromosomes (nt), 20000 - 25000 works well. 
  For Arabidopsis chromosomes in units of nt/1000, 20 - 25 works well.

plot_scale_factor: .25

  The factor to change the size of the dot plot; e.g. 0.5

ids_on_png: 1  

  Print diagonal names on big dot plot?
  
COPYRIGHT

  Copyright (c) 2003, Steven Cannon cann0010@umn.edu

  This script can be used and distributed on the same terms as Perl itself. 

AUTHOR 

  Steven Cannon
  Email cann0010@tc.umn.edu
  copyright Steven Cannon, 2003. All rights reserved.
  You may distribute this code under the same terms as perl itself
  The script may be used and modified freely, but I do request that this 
  copyright notice remain attached to the file. You may modify this 
  module as you wish, but if you redistribute a modified version, 
  please attach a note listing the modifications you have made.


VERSION

 1.17

=cut

=head1 ALGORITHM / GENERAL STRAGETY

Basically, the program works by walking through the array of hits. At
each hit, it checks in the neighborhood for hits that might be "direct repeats"
or "inverted repeats", and chooses those nearest and in the
most favorable positions first. It does this recursively, checking up
to 75 possible positions in the vicinity, and then repeating this
until it comes to the end of the diagonal. Then it starts walking
again from where it started the diagonal. Sensitivity is increased,
and time to process the data is decreased, if the sparse hits are
brought "closer together" by compressing the original matrix. If this
sort of compression is done, then the original coordinates are
recovered at the end. The three output files consist of two dot plots
(big and small, with the big one having the option of names being
added to each diagonal), and a text file with the coordinates of each
diagonal.

Some more specifics: The program reads in a matrix containing coordinates 
and similarity scores. To start, after filtering for hit strength, 
it stores only the hit data and strand orientation, and keeps 
these records in a one-dimensional array (though the numbers are large --
genomic scale, with coordinates in nt or kb). 

   1 2 3 4 5 6 7 8 9 0
 1   1     2         
 2     3         
 3       4     5
 4           6
 5             7
 6               
 7                 8
 8         
 9         9   0
 0
 
These are stored in an array, after first sorting by both dimensions, and 
collapsing these into a string.

To look for diagonals, walk through this array. For each item, check in the
neighborhood, weighting "close" hits higher than more distant ones (see 
notes below on search order and specificity. Strand information is used this
way. Orientation is stored as w (Watson orientation) or c (Crick orientation).
Direct-repeat diagonals are composed of w/w or c/c hit pairs, and indirect-
repeat diagonals are composed of w/c or c/w hit pairs. After a walk through
a candidate diagonal, If the accumulated diagonal is "long enough" (another 
parameter), remove these elements from the array (so we don't find sub-arrays 
later on), and go back to where we left off ($a[1]), and look for another 
diagonal. In looking for diagonals, check recursively down and to the right 
along some set of allowed paths, OR recursively down and to the left.

=head2 Search order and specificity

The @search_order array specifies the order in which to look for next 
elements in the diagonal, in distances from last element. The sensitivity 
paramter is how far to go through the @search_order list (0 indexed, so we 
loop from 0 to <= $sensitivity). Currently, the range is 0 - 75

  | 0  1  2  3  4  5  6  7  8  9
 -+----------------------------- 
 0| X 13 15 21 25
 1|13  1  4  8 30 31 35
 2|15  4  2  5 10 13 31 36
 3|21  8  5  3  7 12 20 32 37
 4|25 30 10  7  6 11 16 21 33
 5|   31 13 12 11  9 15 19 28 34
 6|   35 31 20 16 15 14 18 24 29
 7|      36 32 21 19 18 17 23 27
 8|         37 33 28 24 23 22 26
 9|               34 29 27 26 25

=cut

#------------------------------------------------------------------------------#
#------------------------------- SETUP OPTIONS --------------------------------#

# Set globals. The next items are used in the diagonal-starting and the 
# recursive diagonal-extending subroutines. Out of laziness, I don't want to 
# keep passing all of these as locals.
my ($near_main_diag, $score_column, $score_cutoff, $min_diag_len, $sensitivity, 
    $min_diag_qual, $compress_factor, $use_orientation, 
    $scale_factor, $ids_on_png );
my ($base_dir, $in_subdir, $out_subdir, $debug); # IO stuff
my @ra; # make this large array global; avoids lots of ref. passing in recursion
my @orig_ra; # store unaltered array of compressed hits
my %ra_to_undef;
my $x_start; my $y_start; # starting points of diags.
my $diag; # diagonal in progress
my @diagonal; # holds the final diagonals
my %hsh_name_by_posn; # holds gene names, keyed from position (for genoPix)
my $search_index; # index in this diagonal
my $count_in_diag; # length of diagonal
my $diag_num; # index of the diagonals
my $diag_quality; # basically, average of the @search_order index in a diagonal;
                  # measures the diagonal's density. Lower is better.
my %xy_orig_hash; # used during matrix compression and recovery
my $max_x; my $max_y; 
my @x; my @y;
my ($chr_A, $chr_B);
my @diag_ra;
my $created_output_directory = 0;
my @reported_txt_diag_hits = ();
my $ave_diag_quality; # ... divide $diag_quality by $count_in_diag
my @search_order = qw(11 22 33 12 21 32 23 44 43 34 13 31 55 42 24 45 
54 53 35 25 52 01 10 66 65 56 20 02 46 64 77 76 67 57 75 63 36 47 74 03 30 
88 78 87 86 68 99 40 04 89 98 97 79 58 85 96 69 41 14 26 62 51 15 73 37 38 
83 59 95 61 16 72 27 83 38
); 

##### start IO boilerplate #####
my (@file_list, $log_string, $config_file);
if(!defined($ARGV[0])) { # get config file
  $0 =~ m/(\d+).+/; # grab program number; e.g. 12 from 12_BlastSelf
  my $prog_num = $1;
  die "\n Must supply config file name. Usage e.g.: $0 $prog_num.cfg\n"
} 
else { $config_file = $ARGV[0] }
my $log_file =   "$0.log";
read_config_file(); # get params from config file.

sub read_config_file {
  open (CFG, "< $config_file") or die "can't open $config_file:$!";
  while (<CFG>) {
    unless (m/^\s*[#\s]/) { # don't look at comment or blank lines
      my ($a, $b) = split /:\s*/, $_; # split lines at ':'
      chomp $a; chomp $b;
      unless ($a eq "file_list") { 
        $b =~ s/\s+//g; # strip whitespace
        $log_string .= "  $a:  $b\n";
      }
      if ($a eq "file_list") { 
        @file_list = split /\s/, $b;
        $log_string .= "  $a:  ";
        foreach my $item (@file_list) {
          $item =~ s/\s//; $log_string .= "[$item]  ";
        } $log_string .= "\n";
      }
	  elsif ($a eq "base_dir") { $base_dir = $b }
	  elsif ($a eq "in_subdir") { $in_subdir = $b }
	  elsif ($a eq "out_subdir") { $out_subdir = $b }
	  elsif ($a eq "compress_factor") { $compress_factor = $b }
	  elsif ($a eq "use_orientation") { $use_orientation = $b }
	  elsif ($a eq "scale_factor") { $scale_factor = $b }
	  elsif ($a eq "ids_on_png") { $ids_on_png = $b }
	  elsif ($a eq "score_column") { $score_column = $b }
	  elsif ($a eq "score_cutoff") { $score_cutoff = $b }
	  elsif ($a eq "near_main_diag") { $near_main_diag = $b }
	  elsif ($a eq "min_diag_len") { $min_diag_len = $b }
	  elsif ($a eq "sensitivity") { $sensitivity = $b }
	  elsif ($a eq "min_diag_qual") { $min_diag_qual = $b }
	  elsif ($a eq "debug") { $debug = $b }
	  else { print "\n  skipping unrecognized parameter '$a:  $b'\n" }
    }
  }
  $log_string .= "\n";
  if ($debug) { print "$log_string" }
}
##### end IO boilerplate #####
 
if ($debug) { print "Parameters:\n" }

my $in_dir = catfile($base_dir, $in_subdir);
my $out_dir; # will be created in sub file_setup
my ($out_sm_png, $out_lg_png);

######################### MAIN LOOP ###########################

foreach my $file_in (@file_list) {
  if ($debug) { print "Starting on $file_in\n" }
  @diag_ra = ();
  @ra = ();
  @orig_ra = ();
  @diagonal = ();
  %hsh_name_by_posn = ();
  %xy_orig_hash = (); 
  @x = ();  @y = ();
  my @unsorted = ();
  my @sorted = ();
  @reported_txt_diag_hits = ();
  
  # Open a few files for writing. Currently, I don't check for over-writing.
  file_setup($file_in);
  
  # read parsed blast hits and filter.
  my $unsorted = read_and_filter($file_in);
    if ($debug == 5 ) { print "unsorted:\n"; 
      foreach my $item1 (@$unsorted) { print "$item1\n" }}
  
  # sort hits by second number (x) and then by first (y).
  my $sorted = sort_hits($unsorted);
    if ($debug == 4 ) { print "sorted:\n"; 
      foreach my $item2 (@$sorted) { print "$item2\n" }}
  
  # put hits into an array
  compress_and_make_xy_array($sorted);
  
  # scan for diagonals (go through each hit to see if it might start a diagonal)
  look_for_diags();
  
  # make diagonal coordinates more accessible by putting them 
  # in arrays rather than strings
  parse_sorted_coords();
  
  # print the diagonal coordinates to the text file
  print_diag_coords();
  
  # print the array, and highlight the diagonals on the .png
  plot_array();

  # add numbers to the diagonals on the .png
  plot_array_nms();

  # quit
  print "\nDone with  $file_in\n\n";
  
  close OUT;

}

print "\ndone!\n\n";

############################### SUBROUTINES ####################################

# file_setup: open files, make filehandles, etc.
sub file_setup {
  my $file_in = shift;
  my ($out_coords, $out_pairs);
  my ($dir_file_in, $out_log);
  if ($debug) {print "file_setup\n";}
  $file_in =~ /(\w+(\d+)x(\d+)).+/;
  $chr_A = $2; $chr_B = $3;
  my $param_descriptions = 
  "Parameters for files in $in_dir:
  _sc  score_cutoff    bit score cutoff before diag search $score_cutoff 
  _cf  compress_factor original matrix is compressed by    $compress_factor 
  _l   min_diag_len    min diag length of compressed diag  $min_diag_len 
  _sn  sensitivity     sensitivity (see manpage)           $sensitivity 
  _q   min_diag_qual   min diag quality (see manpage)      $min_diag_qual 
       file            input matrix                        $file_in 
       scale_factor    scales the dot plot                 $scale_factor 
  
  Color key:
       red     both genes are w/w or c/c orientation (or if no w/c info)
       violet  both genes are w/c or c/w orientation
       black   (small plot): diagonal
       green   (large plot): diagonal (numbered to match text output
               in .coords and diagonal_group_*.txt files)
  ";
  my $params = "_sc$score_cutoff"."_cf$compress_factor"."_l$min_diag_len".
               "_sn$sensitivity"."_q$min_diag_qual";
  $dir_file_in = "$in_dir/$file_in";
  
  # Generate a directory to hold the results (only need to do this once):
  unless ($created_output_directory) {
    $out_dir = catfile($base_dir,$out_subdir);
    $out_dir .= $params;
    if ($debug) {print "  Creating   $out_dir\n"}
    system   ( "mkdir $out_dir");
    $created_output_directory = 1;
    $params .= '.log';
    $out_log = catfile($out_dir,$params);
	open (LOG, "> $out_log") or die "can't open out $out_log:$!";
	  if ($debug) {print "  Creating  $out_log\n"}
	print LOG "$param_descriptions\n\n";
	print LOG "$log_string\n";
	print LOG "Program: $0\n";
	print LOG "Version: $VERSION\n";

	my ($day, $month, $year) = (localtime) [3, 4, 5];
	my $yr = $year+1900; 
	my $mo = $month+1; 
	my $dy = $day;

    print LOG "The program was run on $mo/$dy/$yr\n\n"; 
	close LOG;
  }
 
  $out_sm_png = catfile($out_dir,$1); $out_sm_png .= '_sm.png';
  $out_lg_png = catfile($out_dir,$1); $out_lg_png .= '_lg.png';
  $out_coords = catfile($out_dir,$1); $out_coords .= '.coords';
  $out_pairs = catfile($out_dir,"diagonal_group_$1"); $out_pairs .= '.txt';
  
  if ($debug) {print "  Opening   $dir_file_in\n"}
  open (IN, "< $dir_file_in") or die "can't open in $dir_file_in:$!";
  
  if ($debug) {print "  Creating  $out_coords\n"}
  open (OUT, "> $out_coords") or die "can't open out $out_coords:$!";
  
  if ($debug) {print "  Creating  $out_pairs\n"}
  open (PAIRS, "> $out_pairs") or die "can't open out $out_pairs:$!";

  return;
}

# read parsed blast hits and filter. 
sub read_and_filter {
  my $file_in = shift;
  if ($debug) {print "read_and_filter\n";}
  # First num is posn in first chromosome; second is posn in second chrom; 
  # third is bit score.
  my @unsorted = ();
  while (<IN>) {
    chomp;
    my $in = $_;
	$in =~ 
	 /^([^\t]+)\t([^\t]+)\t([^\t]+)\t([^\t]+)\t([^\t]+)\t([^\t]+)\t([^\t]+)/;
	my $gn1 = $1;
	my $psn1 = $2;
	my $gn2 = $3;
	my $psn2 = $4;
	my $sc1 = $5;
	my $sc2 = $6;
	my $sc3 = $7;
	my $scX;    # print "$gn1 $gn2 $psn1 $psn2 $sc3\n";
	if ($score_column == 5) {$scX = $sc1}  # pick the right score cutoff column
	elsif ($score_column == 6) {$scX = $sc2}
	elsif ($score_column == 7) {$scX = $sc3}
	else { die "In the config file, score_column must be 5, 6, or 7\n" }
	if ($scX >= $score_cutoff) {           # ... and use it as a threshold
	  push(@unsorted, "$psn1\t$psn2\t$scX");
	  $hsh_name_by_posn{$psn1} = $gn1;
	  $hsh_name_by_posn{$psn2} = $gn2;
	}      
  }
  close IN;
  return \@unsorted;
}

# sort hits by second number (x) and then by first (y). See Camel book p. 120.
# This is a relatively small array.
sub sort_hits {
  if ($debug) {print "sort_hits\n";}
  my $r_unsorted = shift; 
  my @unsorted = @$r_unsorted;
  my @sorted = ();
  @sorted = map   { $_->[2] }   # map original data onto sorted anonymous array?
      sort  {   $a->[0] <=> $b->[0]   # sort by second (y) element
                or
            $a->[1] <=> $b->[1]    # sort by first (x) element
          }
      map   { [ /(\d+)[wc-]*\t(\d+)[wc-]*/, $_ ] }  # grab coords (& w or c?)
      @unsorted;
  return \@sorted;
}

# put hits into an array keyed on x and y coordinates
sub compress_and_make_xy_array {
  if ($debug) {print "compress_and_make_xy_array\n"}
  my $r_sorted = shift; 
  $max_x = 0; $max_y = 0; 
  foreach my $item (@$r_sorted) {
    my ($x, $y, $sc) = split /\t/,$item;
    if ($debug > 3) {print "($x,$y,$sc) "} # print sorted array of hits.
    
    # Next, store orig coords & w/c for later recovery in sub print_diag_coords
    my $xy_orig = "$x"."_$y"; # may look like 1234222w_567222c or 1234222_567222
    
    # grab coords and orientation
    $x =~ /(\d+)([wc-]*)/; 
    my $x_orig = $1;
    # grab orientation, so see if both hits are (ww or cc) or (wc or cw)
    my $xwc = $2;
    
    $y =~ /(\d+)([wc-]*)/;    
    my $y_orig = $1;
    my $ywc = $2;
    
    $x = int $x_orig/$compress_factor; 
    $y = int $y_orig/$compress_factor;
    my $xy = "$x" . "_$y"; # looks like 1234_567
    # print "[$xy=>$xy_orig] ";

    # Next, we'll hash the new and the compressed coordinates. The sensitivity 
    # of the algorithm depends on the compression factor, but we'll want to 
    # report actual coordinates later, and can't do this just by multiplying 
    # by the compression factor because we've thrown away the decimal portion. 
    # This could still cause a problem: hits very near one another might 
    # recover the same original position: for large c and small x - x', 
    # c*int(x/c) = c*int(x'/c) (repeat this for y; the pairs form the hash key).
    
    $xy_orig_hash{$xy} = $xy_orig;
    
    $max_x = $x unless ($x < $max_x);
    $max_y = $y unless ($y < $max_y);
    unless ($x == $y) {  # exclude the "self" diagonal
      if ($use_orientation == 0 or ($xwc eq 'w' and $ywc eq 'w') 
      or ($xwc eq 'c' and $ywc eq 'c')) {
        $ra[$x][$y] = 1;
        $orig_ra[$x][$y] = 1;
      }
      elsif (($xwc eq 'w' and $ywc eq 'c') or ($xwc eq 'c' and $ywc eq 'w')) {
        $ra[$x][$y] = 2;
        $orig_ra[$x][$y] = 2;
      }
      else { 
        $ra[$x][$y] = 3;
        $orig_ra[$x][$y] = 3;
      }
    }
  } 
  if ($debug >=2) {print "\nmax_x: $max_x; max_y: $max_y\n";} 
}

sub look_for_diags {
  # walk through the array, looking for diagonals
      if ($debug) {print "look_for_diags. For long diags, the" .
        " recursion may be deep (that's OK).\n";
      }
  $diag_num = 0;
  $count_in_diag = 0;
  $diag = "";
  my ($i,$j);
  for ($i = 0; $i < $max_x; $i++) { 
    for ($j = 0; $j < $max_y; $j++) {
      if ($debug > 2 and defined($ra[$i][$j])) {print "\n*$i,$j*";}
	  if (defined($ra[$i][$j])) { # main diag was excluded earlier
        
        # remember where we left off, for after walking the diagonal
		$x_start = $i; $y_start = $j;

	    # start a string containing coordinate pairs
		$diag = "$i $j\t";

        # give the starting point to the recursive algorithms

		# initialize before the recursive "backward diagonal" search
		 $diag_quality = 0;
		 $ave_diag_quality = 0;
		 $search_index = 0;
		 $count_in_diag = 0;
         %ra_to_undef = ();
		        
        extend_direct_diag($i,$j);

		# initialize before the recursive "forward diagonal" search
		 $diag_quality = 0;
		 $ave_diag_quality = 0;
		 $search_index = 0;
		 $count_in_diag = 0;
         %ra_to_undef = ();

        extend_inverse_diag($i,$j);
        
		next;
	  }
	}
  }
}

##### diagonal-extending subroutines #####

# If 0,0 is upper left, end down and to the right (direct duplication)
sub extend_direct_diag {
  if ($debug > 2) {print "_1_";}
  my $k;
  my ($x,$y) = @_;   if ($debug > 2) { print "[$x,$y]" }
  
  # exclude "near the main diagonal" in a chromosome vs. itself
  return($x,$y) if (($chr_A == $chr_B) and abs($x - $y) <= $near_main_diag);   

  # start searching at the @search_order[0], which is $ra($x+1, $y+1)
  while ($search_index < $sensitivity and $ave_diag_quality < $min_diag_qual) {
    my ($i, $j) = split (//, $search_order[$search_index]);
    $search_index++;
	if (defined($ra[$x+$i][$y+$j]) and $ave_diag_quality < $min_diag_qual
	    and ($use_orientation == 0 or $ra[$x+$i][$y+$j] == 1)) 
	{ 	# The "== 1" checks if the hit is consistent with a direct repeat 
	    # (ww or cc); value was determined in sub compress_and_make_xy_array

	  $x+=$i; $y+=$j; $count_in_diag++; 
	  $diag .= "$x $y\t";       if ($debug > 2) {print " ij:$i$j ";}
	  $diag_quality += $search_index**.8;
	  $ave_diag_quality = $diag_quality/$count_in_diag;
	  if ($debug > 2) {
	   printf "(Q:%1.1f/$count_in_diag=%1.1f) ", $diag_quality, $ave_diag_quality;
	  }
	  # Next: pick up hits near the "best diagonal" and remove these from
	  # further consideration. Helps consolidate diagonals. Check behind and
	  # to the left and right of the current location. The "== 1" check is to
	  # see whether the hit is consistent with a direct repeat (ww or cc);
	  # this was determined in sub compress_and_make_xy_array
	  for (my $s = -2; $s < 5; $s++) {
        if (defined($ra[$x+$s][$y-2]) and ($ra[$x+$s][$y-2] == 1 or $use_orientation == 0)) {
          my $loc_x = $x + $s; # we won't change the "starting point" $x
          my $loc_y = $y - 2; # look two rows behind the current position
          $ra_to_undef{$loc_x} = $loc_y; # to remove fr other srchs if long diag
          $diag .= "$loc_x $loc_y\t";  if ($debug > 2) {print"{$loc_x$loc_y}"}
        }
        if (defined($ra[$x+$s][$y-1]) and ($ra[$x+$s][$y-1] == 1 or $use_orientation == 0)) {
          my $loc_x = $x + $s; # we won't change the "starting point" $x
          my $loc_y = $y - 1; # look one row behind the current position
          $ra_to_undef{$loc_x} = $loc_y; # to remove fr other srchs if long diag
          $diag .= "$loc_x $loc_y\t";  if ($debug > 2) {print"{$loc_x$loc_y}"}
        }
        if (defined($ra[$x+$s][$y]) and ($ra[$x+$s][$y] == 1 or $use_orientation == 0)) {
          my $loc_x = $x + $s; # we won't change the "starting point" $x
          my $loc_y = $y; # look in the current row
          $ra_to_undef{$loc_x} = $loc_y; # to remove fr other srchs if long diag
          $diag .= "$loc_x $loc_y\t";  if ($debug > 2) {print"{$loc_x$loc_y}"}
        }
      }	  
	  $search_index = 0;
	  extend_direct_diag($x,$y);
	}
  }
  
  # search is maxed out (end of diag), and string is long enough
  if ($count_in_diag >= $min_diag_len) {
	$x = $x_start; $y = $y_start; 
	my %seen = ();
	# we're at the end of a diagonal. Reset counters to the starting position, 
	# to keep looking for diagonals from there.
	if ($debug > 2) {print "($x,$y)|\n";}
	$diag_num ++;
	my @temp = split ("\t", $diag);
	if ($debug > 2) {print "\n";}
	if ($debug > 1) {print "FWD: ";}
	foreach my $pair (@temp) {
	  unless ($seen{$pair}) {
	    $seen{$pair} = 1;
		$k++;  # increment first, to save 0 for array size
		if ($debug > 1) {print "{$pair}";}
		my ($s, $t) = split(/\s/, $pair);
		undef($ra[$s][$t]); # remove these diagonal items from further searches
		$diagonal[$diag_num][$k] = $pair;
		$diagonal[$diag_num][0] = $k;  # save array size in 0th element
      }
	}
	# Next routine should remove "near" diagonal items from further searches.
	foreach my $key (keys %ra_to_undef) {
	  my $s = $key;
	  my $t = $ra_to_undef{$key};
	  undef($ra[$s][$t]); # remove these diagonal items from further searches
	  # print "[$s,$t]"
	}
	if ($debug > 1) {print "\n";}
	$count_in_diag = 0;
	$diag = ""; 
	$k = 0;
	return($x,$y)
  }
}


# If 0,0 is upper leftext, end down and to the left (inverse duplication)
sub extend_inverse_diag {
  
  # check in the direction go (x+1,y-1), or more generally: (x+i,y-j)
  # (had been checking (x-i,y+j) until v.0.99 )

  if ($debug > 2) {print "_2_";}
  my $k;
  my ($x,$y) = @_;   if ($debug > 2) {print "[$x,$y]";}

  # exclude "near the main diagonal" in a chromosome vs. itself
  return($x,$y) if (($chr_A == $chr_B) and abs($x - $y) <= $near_main_diag);   

  # start searching at the @search_order[0], which is $ra($x+1, $y-1)
  while ($search_index < $sensitivity and $ave_diag_quality < $min_diag_qual) {
    my ($i, $j) = split (//, $search_order[$search_index]);
    $search_index++;
	if (defined($ra[$x+$i][$y-$j]) and $ave_diag_quality < $min_diag_qual
	    and $y-$j > 0 and ($use_orientation == 0 or $ra[$x+$i][$y-$j] == 2)) 
	{   # $y-$j > 0: in inverse search, skip if out of bounds.
	 	# The "== 2" checks if the hit is consistent with an inverted repeat 
	    # (wc or cw); this was determined in sub compress_and_make_xy_array

	  $x+=$i; $y-=$j; $count_in_diag++; 
	  $diag .= "$x $y\t";       if ($debug > 2) {print " ij:$i","-$j ";}
	  $diag_quality += $search_index**.8;
	  $ave_diag_quality = $diag_quality/$count_in_diag;
	  if ($debug > 2) {
	   printf "(Q:%1.1f/$count_in_diag=%1.1f) ", $diag_quality, $ave_diag_quality;
	  }
	  # Next: pick up hits near the "best diagonal" and remove these from
	  # further consideration. Helps consolidate diagonals. Check behind and
	  # to the left and right of the current location. The "== 2" check is to
	  # see whether the hit is consistent with an inverted repeat (wc or cw);
	  # this was determined in sub compress_and_make_xy_array
	  for (my $s = -2; $s < 5; $s++) {
        if (defined($ra[$x+$s][$y+2]) and ($ra[$x+$s][$y+2] == 2 or $use_orientation == 0)) {
          my $loc_x = $x + $s; # we won't change the "starting point" $x
          my $loc_y = $y + 2; # look two rows behind the current position
          $ra_to_undef{$loc_x} = $loc_y; # to remove fr other srchs if long diag
          $diag .= "$loc_x $loc_y\t";  if ($debug > 2) {print"{$loc_x$loc_y}"}
        }
        if (defined($ra[$x+$s][$y+1]) and ($ra[$x+$s][$y+1] == 2 or $use_orientation == 0)) {
          my $loc_x = $x + $s; # we won't change the "starting point" $x
          my $loc_y = $y + 1; # look one row behind the current position
          $ra_to_undef{$loc_x} = $loc_y; # to remove fr other srchs if long diag
          $diag .= "$loc_x $loc_y\t";  if ($debug > 2) {print"{$loc_x$loc_y}"}
        }
        if (defined($ra[$x+$s][$y]) and ($ra[$x+$s][$y] == 2 or $use_orientation == 0)) {
          my $loc_x = $x + $s; # we won't change the "starting point" $x
          my $loc_y = $y; # look in the current row
          $ra_to_undef{$loc_x} = $loc_y; # to remove fr other srchs if long diag
          $diag .= "$loc_x $y\t";  if ($debug > 2) {print"{$loc_x$loc_y}"}
        }
      }	  
	  $search_index = 0;
      extend_inverse_diag($x,$y);
	}
  }
  
  # search is maxed out (end of diag), and string is long enough
  if ($count_in_diag >= $min_diag_len) {
	$x = $x_start; $y = $y_start; 
	my %seen = ();
	# we're at the end of a diagonal. Reset counters to the starting position, 
	# to keep looking for diagonals from there.
	if ($debug > 2) {print "($x,$y)|\n";}
	$diag_num ++;
	my @temp = split ("\t", $diag);
	if ($debug > 2) {print "\n";}
	if ($debug > 1) {print "REV: ";}
	foreach my $pair (@temp) {
	  unless ($seen{$pair}) {
	    $seen{$pair} = 1;
		$k++;  # increment first, to save 0 for array size
		if ($debug > 1) {print "{$pair}";}
		my ($s, $t) = split(/\s/, $pair);
		undef($ra[$s][$t]); # remove these diagonal items from further searches
		$diagonal[$diag_num][$k] = $pair;
		$diagonal[$diag_num][0] = $k;  # save array size in 0th element
	  }
	}
	# Next routine should remove "near" diagonal items from further searches.
	foreach my $key (keys %ra_to_undef) {
	  my $s = $key;
	  my $t = $ra_to_undef{$key};
	  undef($ra[$s][$t]); # remove these diagonal items from further searches
	  # print "{$s,$t}"
	}
	if ($debug > 1) {print "\n";}
	$count_in_diag = 0;
	$diag = ""; 
	$k = 0;
	return($x,$y)
  }
}

##### text output subroutines #####

sub parse_sorted_coords {
  # make diag coords accessible by putting them in arrays rather than strings
      if ($debug) {print "parse_sorted_coords\n";}
  my $m = 0; my $k = 0;
  for ($k = 1; $k <= $diag_num; $k++) {  # go from first diag to the last
    for ($m = 1; $m <= $diagonal[$k][0]; $m++) { # go through all pairs in diag
      $diagonal[$k][$m] =~ /(\d+) (\d+)/;      # parse the pairs
      $x[$k][$m] = $1;      # ... and put into an array 
      $y[$k][$m] = $2;
    }
  }
}

sub print_diag_coords {
  # print diag coordinates, starting from first (x) chromosome 
  # (i.e. coords look like (x,y))
  if ($debug) {print "print_diag_coords\n";}
  print OUT
   "chr_A\t$chr_A\nchr_B\t$chr_B\nmax_A\t$max_x\nmax_B\t$max_y\n" . 
   "compress_factor\t$compress_factor\n";
  print OUT
   "diag_names\t";
  for (my $k = 1; $k <= $diag_num; $k++) {   # the diagonal numbers
    print OUT $k;
    unless ($k == $diag_num) { print OUT ":" } # don't print ':' after last item
  }
  print OUT "\n";
  my $m = 0; my $k = 0;
  my ($recovered_xwc, $recovered_ywc, $recovered_x, $recovered_y, %seen);
  for ($k = 1; $k <= $diag_num; $k++) {  # the diagonal number
    for ($m = 1; $m <= $diagonal[$k][0]; $m++) {
      # What comes next is ugly, but basically, $xy_orig_hash{$xy_key}
      # recovers the non-compressed x_orig and y_orig using the 
      # transformed values: $x = int $x_orig/$compress_factor & comparable $y. 
      # Potential for error: two $x, $y pairs could be rounded to recover
      # the same $x_orig and $y_orig.
      my $xy_key = "$x[$k][$m]" . "_$y[$k][$m]";
      if (defined($xy_orig_hash{$xy_key})) {
        
        #incoming: 1234222w_567222c or 1234222_567222

        $xy_orig_hash{$xy_key} =~ /((\d+)[wc-]*)_((\d+)[wc-]*)/;
        
        $recovered_xwc = $1; # 1234222w or 1234222 if missing w
        $recovered_x   = $2; # 1234222

        $recovered_ywc = $3; # 567222c or 567222 if missing c
        $recovered_y   = $4; # 567222

        # Next: basically for debugging, to later plot and
        # see how lossy the algorithm was
        my $reported_txt_diag_hit = 
         int($recovered_x/$compress_factor) . "_" . 
         int($recovered_y/$compress_factor);
        push @reported_txt_diag_hits, $reported_txt_diag_hit;
        
 		# @diag_ra is used later to print diag & identify class of diag
		$diag_ra[$x[$k][$m]][$y[$k][$m]]=$k; 

		#print OUT "$recovered_x:$recovered_y\t"; 
		print OUT "$recovered_xwc:$recovered_ywc\t";
		# output looks like 
		# ID_1	ID_2	5.2.Group3
		print PAIRS $hsh_name_by_posn{$recovered_xwc} . "\t" .
		            $hsh_name_by_posn{$recovered_ywc} . "\t" .
		            "$chr_A.$chr_B.group$k\n";
      }
    }
    print OUT "\n";
  }
}


##### graphics subroutines #####

sub plot_array {
  if ($debug) {print "plot_array\n";}

  open (PNG, "> $out_sm_png") or die "can't open out $out_sm_png:$!";
  binmode PNG;
	if ($debug) {print "  Creating  $out_sm_png\n"}

  my ($x, $y);
  my $s = $scale_factor;
  my $im1 = new GD::Image($max_x*$s,$max_y*$s) or die;   # create a new image
  my $white = $im1->colorAllocate(255,255,255);
  my $black = $im1->colorAllocate(0,0,0); 
  my $gray = $im1->colorAllocate(100,100,100); 
  my $red1 = $im1->colorAllocate(255,100,100); 
  my $red2 = $im1->colorAllocate(255,0,0); 
  my $violet1 = $im1->colorAllocate(255,150,250); 
  my $green = $im1->colorAllocate(0,255,0);    
  $im1->rectangle(0,0,$max_x*$s-1,$max_y*$s-1,$green);
  
  for ($x = 0; $x < $max_x; $x++) { # plot diagonals
    for ($y = 0; $y < $max_y; $y++) {
 
      if (defined($orig_ra[$x][$y])) { 
        if ($orig_ra[$x][$y] == 1) { # cc or ww
          $im1->setPixel(int($x*$s),int($y*$s),$red1);
        }
        elsif ($orig_ra[$x][$y] == 2) { # cw or wc
          $im1->setPixel(int($x*$s),int($y*$s),$violet1);
        }
        else { # shouldn't get here, unless we were missing w or c info
          $im1->setPixel(int($x*$s),int($y*$s),$red2);
        }
      }
      
      if (defined($diag_ra[$x][$y]))   {   # plot the diagonals on top
        $im1->setPixel(int($x*$s),int($y*$s),$black);
        $im1->setPixel(int(($x+1)*$s),int($y*$s),$black);
        $im1->setPixel(int($x*$s),int(($y+1)*$s),$black);
        $im1->setPixel(int(($x+1)*$s),int(($y+1)*$s),$black);
      }
    }
  }
    
  print PNG $im1->png;
  close PNG;
}


sub plot_array_nms {
  if ($debug) {print "plot_array_nms\n";}

  open (PNG_NM, "> $out_lg_png") or die "can't open out $out_lg_png:$!";
	if ($debug) {print "  Creating  $out_lg_png\n"}
  binmode PNG_NM;

  my $im2 = new GD::Image($max_x,$max_y) or die;  # create a new image
  my ($x, $y, $m, $k);
  my $white = $im2->colorAllocate(255,255,255);
  my $black = $im2->colorAllocate(0,0,0); 
  my $gray = $im2->colorAllocate(200,200,200); 
  my $red2 = $im2->colorAllocate(255,0,0); 
  my $violet1 = $im2->colorAllocate(255,150,250); 
  my $violet2 = $im2->colorAllocate(150,0,255); 
  my $blue = $im2->colorAllocate(0,0,255); 
  my $green = $im2->colorAllocate(0,255,0);    
  for ($x = 0; $x < $max_x; $x++) {
    for ($y = 0; $y < $max_y; $y++) {

      if (defined($orig_ra[$x][$y])) {
        if ($orig_ra[$x][$y] == 1) { # cc or ww
          $im2->setPixel($x,$y,$red2);
        }
        elsif ($orig_ra[$x][$y] == 2) { # cw or wc
          $im2->setPixel($x,$y,$violet2);
        }
        else { # shouldn't get here, unless we were missing w or c info
          $im2->setPixel($x,$y,$gray);
        }
      }
      
      # print a green background for the diagonals
      if (defined($diag_ra[$x][$y])) {
        $im2->setPixel($x,$y,$blue);
        $im2->setPixel($x+1,$y,$green);
        $im2->setPixel($x,$y+1,$green); 
        $im2->setPixel($x+1,$y+1,$green);
        $im2->setPixel($x+1,$y-1,$green);
        $im2->setPixel($x-1,$y,$green);
        $im2->setPixel($x,$y-1,$green);
        $im2->setPixel($x-1,$y-1,$green);
        $im2->setPixel($x-1,$y+1,$green);
      }
      
    }
  }

  # plot all hits on top
  for ($x = 0; $x < $max_x; $x++) {
    for ($y = 0; $y < $max_y; $y++) {
     
      # ... then the hit itself
      if (defined($orig_ra[$x][$y])) {
        if ($orig_ra[$x][$y] == 1) { # cc or ww
          $im2->setPixel($x,$y,$red2);
        }
        elsif ($orig_ra[$x][$y] == 2) { # cw or wc
          $im2->setPixel($x,$y,$violet2);
        }
        else { # shouldn't get here, unless we were missing w or c info
          $im2->setPixel($x,$y,$violet1);
        }
      }
    }
  }
  
  # print diagonal names on the plot
  if ($ids_on_png) {
	for ($k = 1; $k <= $diag_num; $k++) {
	  $m = 1;
	  $x = $x[$k][$m];
	  $y = $y[$k][$m];
	  $im2->string(gdTinyFont,$x,$y+7,$diag_ra[$x][$y],$black); # diag class at top
	}
  }
  print PNG_NM $im2->png;
  close PNG_NM;
}


# VERSION HISTORY
# 
# v.0.010 Jan 15'03 San Diego; rewrite of 01_diag_prot_mtx_7.3. Goals: cleaner,
# shouldn't rely on matrix compression, more efficiently traverse hits when
# looking for diagonals. This version: POD and basic framework are in place.
# v.0.013 Jan 17'03 works basically for walking through direct repeats, and
#    is faster than 01_diag_prot_mtx ... (but see below, at v.016)
# v.0.014 Jan 17'03 make the recursive walk more flexible & sensitive (in loop)
# v.0.015 Jan 17'03 allow vertical steps.
# v.0.016 Jan 18'03 test parameters with real data. Without compression, 
#  not sensitive enough.
# v.0.017 Jan 18'03 Add compression. Sadly, increase from order N to N^2 in 
#  look_for_diags. Probably would be a better way using hash instead of array.
#  This version works basically -- but graphics isn't added back in yet.
# v.0.018 Jan 18'03  Add back the graphics routines. Both plots are order N^2
# v.0.90 Jan 18'03 Rename from diagomizer to diag_hunter. Works nicely.
#         With Athal gene data (1k resolution), decent parameters are:
#         sc450_cf20_l6_sn66_q9 (defaults stored with this version).
# v.0.91 Jan 21'03 Default param changes (for Athal data, with 
#        positions divided by 1000: sc400_cf15_l5_sn66_q10
#        A change in the recursion: pick up hits near the "best diagonal" 
#        and remove these from further consideration. Helps consolidate diags.
#        Minor documentation edits. 
# v.gp_0.91 Jan 22'03 Default params are changed to work with genoPix 
#   data and data structure. sc50_cf25000_l5_sn66_q10 Equivalent to:
#    -score_cutoff=50 -compress_factor=25000 -verbose_input
#    -min_diag_len=5 -min_diag_qual=10 -file_list='chr1x1.Ath_verbose'
# v.gp_0.92 Jan 22'03 Prints two text output files: one with genex\ngeney\n
#     and one with genex\tgeney\n
# v.gp_0.93 Jan 24'03 Change text output to work with orthofinder-finish in .931
# v.0.931 Jan 26-30'03 Change IO to use a config file, to allow storage of 
#     different default parameters separate from the program. Remove most 
#     command line options. Change -man and -help behavior.  
#     Improve the way small diagonals are merged, by adding
#     off-diagonal elements to the building diagonal, and removing these 
#     from consideration in later diagonal searches. Start a directory 
#     structure for a little suite of programs called OrthoParaMap:
# v.0.932 Jan 30'03 Make production of png file optional. Test with both input
#   formats. TO DO: change input format at read_and_filter
#   Added %orig_ra so that all hits are printed (before, those that were part
#   of aborted diagonals were not printed).
# v.0.933 Jan 31'03 minor changes to the search order matrix (~flattened it)
# v.0.934 Feb 1'03 Minor changes to documentation
# v.0.935 Feb 2'03 Remove -verbose option. Simplify input options. Add 51 15
#   and increase sensitivity max to 75.
#   Stop undef'ing nearby hits. This increases the number of diagonals
#   reported (diags aren't collapsed as well), but overall sensitivity is 
#   improved because these are kept in the pool for later diags.
# v.0.936 Feb 2'03 Clear out arrays in main loop to allow multiple infiles
# v.0.937 Feb 3'03 Resurrect the code to combine diagonals (by undefing hits 
#   near a diagonal -- but now *only* if the diagonal is good enough to keep).
#   This works well. Also change diagonal color on large png output. 
#   TO DO: shift from arrays to hashes (will be faster)
# v.0.938 Feb 4'03 Bug fix (typo): commpress_factor -> compress_factor
# v.0.939 Feb 10'03 Fix a serious bug: 
#   *._pairs.txt file often contained wrong pairs, because I was storing 
#   original x and y values from hits by keying off of the compressed x and y
#   separately (really stupid). This version fixes this by keying off x_y.
#   Also removed redundantly reported hits, and plot the hits that actually 
#   get reported in the text output (the algorithm may be lossy, depending 
#   on the level of compression). This is reported using
#   @reported_txt_diag_hits.
# v.0.940 Feb 12'03 Small changes in documentation.
# v.0.941 Feb 18'03 Specify Perl license. 
# v.0.95 Feb 19'03
#   Beta release version, with DiagHunter_0.95.pl, on web sites
#   http://www.atgc.org/GenoPix_2D_Plotter and
#   http://www.tc.umn.edu/~cann0010
#   ----
#   Change output suffix from .diag to .hits
#   -Make regex to "hits" file more flexible.
#   -Append parameter values to output directory (and create directory)
#    rather than appending to filenames.
#   -Change from pairs.txt to diags.txt
# v.0.96 Feb 21'03
#   Fix directory naming bug introduced in .95. Some documentation edits.
#   Re-test with genoPix2D
# v.0.961 Feb 23'03
#   Start to incorporate orientation (intermediate version: parses and stores
#   w and c info in $xy_orig_hash{$xy_key}
# v.0.962 Feb 23'03
#   How to use w/c information? In any diagonal, we should see e.g.
#   ww ww cc ww cc cc ww       (same_phase; direct repeat)       OR
#   wc wc cw cw cw wc cw wc    (opposite_phase; inverted repeat)
#   This works well in this version. Also, I changed Log output (only one log
#   per new output directory).
# v.0.97 Feb 23'03
#   Make w/c optional (though w/c is now recommended). Works with w/c data from 
#   BlastAndParse_0.04.pl
# v.0.97 Feb 23'03
#   Make w/c optional (though w/c is now recommended). Works with w/c data from 
#   BlastAndParse_0.04.pl
# v.0.98 Feb 26'03
#   in *_diags.txt files, indicate which chromosomes were involved in  the 
#   duplications -- for example, in file "chr5x2_diags.txt",
#   lines have the form  "ID_1  ID_2  5.2.Group3" (tab-separated).
#   Also changed the filenames to diagonal_group_*.txt
#   Bug fix: changed my $loc_x = $x - $s to my $loc_x = $x + $s in 
#   extend_inverse_diag (should help consolidate inverse diagonals)
# v.0.99 Mar 4'03
#   change search order in inverse diagonals. Major improvement in inverse diag
#   consolidation. Program had been checking (x-i,y+j).
#   Now, program checks in the direction go (x+1,y-1), or more generally: 
#   (x+i,y-j). This version was posted to A. Kozik's site and mine.
# v.0.991 Mar 11-?'03
#   minor cosmetic changes 
# v.1.10 May'03 Move documentation down. Add a check for config files in this 
#   directory. Test with example Arabidopsis and mouse/human data.
# v.1.11 Jun03'03 Minor stylistic changes to make program consistent with
#   others in the suite
# v.1.12 Jun07'03 Simplify IO. Take config file in via ARGV instead of getopt.
#   This required a lot of changes, including in the config options.
# v1.15 June07'03 re-version for consistency with other OPM programs and
#    planned release
# v1.16 June15'03 fix a directory bug
# v1.17 July21'03 Use File::Spec::Functions to increase portability. 
#        Made GD a requirement.

