#!/usr/bin/perl -w

my $prog_name = "combine_matrix_coords.pl";
$VERSION = 0.06;

use strict;
use File::Spec::Functions; # handle path separators on different platforms
use Getopt::Long;
use Pod::Usage;

# Copyright (c) 2003, Steven Cannon 
# cann0010@umn.edu

# This module can be used and distributed on the same terms as Perl itself. 
 
# POD documentation
#------------------------------------------------------------------------------#
=pod BEGIN

=head1 NAME

 combine_matrix_coords.pl

=head1 SYNOPSIS

  perl combine_matrix_coords.pl -coords_in='../Matrix/ath_ncbi.coords' \
                -matrix_in='../Matrix/ath_ncbi.matrix' -out_dir='DH_hits' -n=5
  
  Required parameters:
    -c = -coords_in  (directory path and filename; may be relative)
    -m = -matrix_in  (directory path and filename; may be relative)
    -o = -out_dir    (directory path; may be relative)
    -n = -num_chromosomes 
  Optional:
    -S = -Symmetrical (convert lower-triangular matrices to square matrices)
    -Co = -Column_with_score (use 3rd or 7th column; 7th is default)
    -Mu = -Mult_factor (1 or 1000000; 1 is default, matching col. 6)
  
Description:
 
  Works with Alexander Kozik's genoPix2D ".coords" and ".matrix" files to produce
  input files readable by the DiagHunter program. The genoPix *.coords data 
  has 9 fields, and looks like:
    1	NP_171610.1	0.007729	C	black		7729	At1g01020	15223281
   or with a dummy place-holder in the 6th field:
    1	NP_171610.1	0.007729	C	black	1	7729	At1g01020	15223281
   The names and indexes are:
    chr	GenbankID	pos_Mb  	W/C	color	?	pos_nt	ID______	Genbank#
    one	two     	three   	4	five	6	seven	eight   	nine
    [0]	[1]     	[2]     	[3]	[4] 	[5]	[6]  	[7]       	[8]
   The fourth column contains orientation data (W=Watson, 5'-3'; C=Crick, 3'-5')
   
  ath_ncbi.matrix
    NP_171609.1	NP_190522.1	0.42	24	156
    NP_171609.1	NP_680217.1	0.4	23	161
    NP_171609.1	NP_564410.1	0.42	21	131

  Output format is like this:
   
    gene name	position	gene name	pos.	####	##	hit length  
    NP_189740.1	12969550c	NP_671757.1	11861c	0.72	100	441
    NP_189780.1	13483272c	NP_671757.1	11861c	0.82	100	335
    NP_189740.1	12969550c	NP_671758.1	15320c	0.63	100	569
  The position fields also cary orientation information, in 'w' or 'c'.

  Note that in ath_ncbi.coords, coordinates are given in two forms: Mb and nt.
  The default is to use the seventh column (coordinates in nt). For example, 
  you might specify  -Column_with_score=3 -Mult_factor=1000000 or 
  -Column_with_score=7 -Mult_factor=1
  
  Currently, combining the .matrix and .coords doesn't produce information 
  about gene orientation. You can produce an output file with gene orientation,
  and use this to make a stronger DiagHunter search, using the BlastAndParse
  script at http://www.tc.umn.edu/~cann0010 .

Author:
 
   Copyright (c) 2003, Steven Cannon 
   cann0010@umn.edu

   This module can be used and distributed on the same terms as Perl itself. 

=cut

#------------------------------- SETUP OPTIONS --------------------------------#
my $help;
my $man;
my $coords_in;  # '../Matrix/ath_ncbi.coords'
my $matrix_in;  # '../Matrix/ath_ncbi.matrix'
my $out_dir;    # '../DH_hits'
my $num_chromosomes;  # 3
my $Symmetrical = 0; # 1 or 0
my $Column_with_score = 7;
my $Mult_factor = 1;

my %options = (
	   "help|h"       => \$help,
	   "coords=s"     => \$coords_in, # '../Matrix/ath_ncbi.coords'
	   "matrix=s"     => \$matrix_in, # '../Matrix/ath_ncbi.matrix'
	   "num_chroms=s" => \$num_chromosomes, # '../DH_hits'
	   "out_dir=s"    => \$out_dir,    # 3 
	   "Symmetrical"  => \$Symmetrical, # if on, make chr x self a square matrix
	   "Column_with_score=s" => \$Column_with_score, # 7 or 3
	   "Mult_factor=s"  => \$Mult_factor # 1 or 1000000
	  );

################################## MAIN ########################################

GetOptions(%options);
pod2usage(-exitstatus => 0) if $help;

die "\nFour parameters are required. Sample usage:\n" .
  "perl $0 -coords='../Matrix/ath_ncbi.coords' -matrix=" .
  "'../Matrix/ath_ncbi.matrix' -out_dir='DH_hits' -num_chroms=5\n" . 
  "  On DOS, you will need something like:\n" .
  "perl $0 -coords='..\\\\Matrix\\\\ath_ncbi.coords' -matrix=" .
  "'..\\\\Matrix\\\\ath_ncbi.matrix' -out_dir='DH_hits' -num_chroms=5\n" . 
  "  or perl $0 -help | more  for more information. Optional parameters: \n" .
  "  -Symmetrical    to make chrrom A x chrrom A a square matrix\n" .
  "  -Column_with_score  use 3rd or 7th column; 7th is default\n" .
  "  -Mult_factor    1 or 1000000; 1 is default, matching column 7\n\n"
  unless ($coords_in and $matrix_in and $num_chromosomes and $out_dir);

# print parameters
if ( $Symmetrical) { print "-S in effect, so make chrA x chrA Symmetrical\n" }
else { print "-S not in effect, so don't make chrA x chrA Symmetrical\n" }
if ( $Column_with_score) { print "-Column_with_score is $Column_with_score\n" }
if ( $Mult_factor) { print "-Mult_factor is $Mult_factor\n" }

for (my $chromA = 1; $chromA <= $num_chromosomes; $chromA++) {
  for (my $chromB = 1; $chromB <= $chromA; $chromB++) { # less redundancy, or...
  #for (my $chromB = 1; $chromB <= $num_chromosomes; $chromB++) { # all combos
    open (COORDS, "< $coords_in") or die "can't open $coords_in:$!";
    open (MATRIX, "< $matrix_in") or die "can't open $matrix_in:$!";

    print "($chromA $chromB)\n";
  
    # format: geneID	position	geneID	position	len
    
    my $diagonals_out = catfile($out_dir, "chr$chromA");
    $diagonals_out .= "x$chromB.hits";
    
    print "  creating $diagonals_out\n";
    open (OUT,   "> $diagonals_out") or die "can't open $diagonals_out:$!";
    
    # Read gene / coordinate pairs from .coords file, and put into a hash
    my ($gene_posA_ref, $gene_posB_ref) = get_coords($chromA, $chromB);
    
    # Read .matrix file, and print out gene/coordinate/gene/coordinate/len
    read_matrix_and_hash_coords($gene_posA_ref,$gene_posB_ref,$chromA,$chromB);
    close COORDS; close MATRIX;
  }
  print "\n";
}

print "Done!\n";

############################## SUBROUTINES #####################################

sub get_coords {
  print "  get_coords ...\n";
  my ($chromA, $chromB) = @_;
  # Read gene / coordinate pairs from .coords file, and put into a hash
  my %gene_posA = (); # will hold the gene / coordinate pairs from .coords
  my %gene_posB = (); # will hold the gene / coordinate pairs from .coords

  my $position;
  while (<COORDS>) {
    # the .coords data has 9 fields, and looks like 
    #  1	NP_171610.1	0.007729	C	black		7729	At1g01020	15223281
    # ... or with a dummy place-holder in the 6th field:
    #  1	NP_171610.1	0.007729	C	black	1	7729	At1g01020	15223281
    # The names and indexes are:
    #  chr	ID______	pos_Mb  	WC	color	?	pos_nt	ID______	ncbi
    #  one	two     	three   	4	five	6	seven	eight   	nine
    #  [0]	[1]     	[2]     	[3]	[4] 	[5]	[6]  	[7]       	[8]
    chomp;
    my @coord_records = split /\t/, $_ ;
    my $chr = $coord_records[0];    # print "$chr ";
    my $ID = $coord_records[1];     # print "$ID ";
    my $pos_Mb = $coord_records[2]; # print "$pos_Mb ";
    my $wc = lc($coord_records[3]); # print "$wc "; # Watson/Crick orientation
    my $pos_nt = $coord_records[6]; # print "$pos_nt ";
    # print join '_', @coord_records, "\n";
    
    if ($Column_with_score == 3) {$position = $pos_Mb * $Mult_factor . $wc}
    elsif ($Column_with_score == 7) {$position = $pos_nt * $Mult_factor . $wc}
    else {print "column does not contain score\n"}
    
    if ($chr == $chromA) { $gene_posA{$ID} = $position}
    elsif ($chr == $chromB) { $gene_posB{$ID} = $position}
  }
  return (\%gene_posA, \%gene_posB);
}

sub read_matrix_and_hash_coords {
  print "  read_matrix_and_hash_coords ...\n";
  my ($gene_posA_ref,$gene_posB_ref,$chromA,$chromB) = @_;
  my %gene_posA = %$gene_posA_ref;
  my %gene_posB = %$gene_posB_ref;
  while (<MATRIX>) {
    $_ =~ /^([^\t]+)\t([^\t]+)\t([^\t]+)\t(\d+)\t(\d+)/;
    my $gene1 = $1;  #  print "$gene1 ";
    my $gene2 = $2;  #  print "$gene2 ";
    my $pos_Mb = $3; #  print "$pos_Mb ";
    my $xxx = $4;    #  print "$xxx ";
    my $len = $5;    #  print "$len\n";
    
    if ($chromA == $chromB) { # making matrix of chrom x self
	  if (exists($gene_posA{$gene1}) and exists($gene_posA{$gene2})) {
		print OUT "$gene1\t$gene_posA{$gene1}\t" . 
				  "$gene2\t$gene_posA{$gene2}\t$pos_Mb\t$xxx\t$len\n";
		print OUT "$gene2\t$gene_posA{$gene2}\t" . 
				  "$gene1\t$gene_posA{$gene1}\t$pos_Mb\t$xxx\t$len\n";
	  }	
	}
	else { # making matrix of chromA x chromB
	  if (exists($gene_posA{$gene1}) and exists($gene_posB{$gene2})) {
		print OUT "$gene1\t$gene_posA{$gene1}\t" . 
				  "$gene2\t$gene_posB{$gene2}\t$pos_Mb\t$xxx\t$len\n";
	  }
	  elsif (exists($gene_posA{$gene2}) and exists($gene_posB{$gene1})) {
		print OUT "$gene2\t$gene_posA{$gene2}\t" . 
				  "$gene1\t$gene_posB{$gene1}\t$pos_Mb\t$xxx\t$len\n";
	  }
    }
  }
  return 1;
}

######################################################
=item
Versions:
v.0.012 Feb2'03 works to produce single files
v.0.013 Feb2'03 works to produce a set of files for each chrom. combination
v.0.014 Feb2'03 Add Pod and Getopt
v.0.015 Feb3'03 option to make chr x self Symmetrical (square rather than 
         a triangular matrix)
v.0.016 Feb12'03 add options Column_with_score and Mult_factor
v.0.02 Feb19'03 
         Beta release version, with DiagHunter_0.95.pl, on web sites
         http://www.atgc.org/GenoPix_2D_Plotter and
         http://www.tc.umn.edu/~cann0010
         ----
         Specify Perl license. 
         Change output suffix from .diag to .hits
          Change default $Mult_factor = 1000000
v.0.03 Feb20'03
         Fix pos_nt pos_Mb problem (I had repeated $pos_Mb), and change
         default -Mult_factor back to 1 (the problem was with pos_Mb)
v.0.04 Feb24'03
         Minor documentation changes
v.0.05 Feb26'03
         Pick up orientation information from the .coords file.
v.0.06 July23'03: add File::Spec::Functions for portability
         
=cut